// ----------------------------------------------------
// Made by: Peepss
// Warning! Do not use this code without my permission.
// ----------------------------------------------------

import config from "./config.js";

import {
    world,
    system
} from "@minecraft/server";
import backpackEventManager from "./customEvents/BackpackEventManager.js";


world.afterEvents.worldInitialize.subscribe(() => {
    world.gameRules.showTags = false;
});

system.runInterval(() => {
    try {
        for (const player of world.getPlayers()) {
            const playerInventory = player.getComponent("inventory").container;
            const selectedItem = playerInventory.getItem(player.selectedSlotIndex);
            const selectedItemData = selectedItem.getDynamicProperty("itemData");

            //If player dies with the backpack in hand, this will prevent the backpack from returning to their inventory with the lockMode "slot".
            for (let slot = 0; slot < playerInventory.size; slot++) {
                const currentItem = playerInventory.getItem(slot);

                if (currentSlot) {
                    if (currentItem.lockMode == "slot" && slot !== player.selectedSlotIndex) {
                        currentItem.lockMode = "none";
                        playerInventory.setItem(slot, currentItem);
                        break;
                    }
                }
            }

            //Applying an ID to the backpack.
            if (selectedItem && config.backpacksTypeIds.includes(selectedItem.typeId)) {
                if (!selectedItemData) {
                    const customId = Math.floor(Math.random() * 9000) + 1000;
                    const itemDataInterface = {
                        typeId: selectedItem.typeId,
                        customId: customId
                    }

                    const itemData = JSON.stringify(itemDataInterface);

                    selectedItem.setDynamicProperty("itemData", itemData);
                    playerInventory.setItem(player.selectedSlotIndex, selectedItem);
                }
            }
        }
    } catch { }
}, 0);

// ---------------------------------
// Events from BackpackEventManager
// ---------------------------------
backpackEventManager.on("release", ({ player, item }) => {
    const playerInventory = player.getComponent("minecraft:inventory").container;

    for (let slot = 0; slot < playerInventory.size; slot++) {
        const currentItem = playerInventory.getItem(slot);

        if (currentItem) {
            let parsedCurrentItemData = null;
            const currentItemData = currentItem.getDynamicProperty("itemData");
            if (currentItemData) {
                try {
                    parsedCurrentItemData = JSON.parse(currentItemData);
                } catch (e) {
                    console.warn(`Erro ao analisar os dados do item no slot ${slot}:`, e);
                }
            }

            const currentCustomId = parsedCurrentItemData?.customId || null;

            if (currentCustomId === item.customId || config.prohibitedItems.includes(currentItem.typeId) || !currentItemData) {
                //If it's a locked item, this will unlock it.
                currentItem.lockMode = "none";
                playerInventory.setItem(slot, currentItem);
            }
        }
    }

    //Play sound effects.
    player.playSound("bundle.remove_one", { pitch: 1.0, volume: 1.0 });
});

backpackEventManager.on("hold", ({ player, item }) => {
    const playerInventory = player.getComponent("inventory").container;
    const selectedSlotItem = playerInventory.getItem(player.selectedSlotIndex);

    if (selectedSlotItem) {
        //Locks the backpack into the current slot.
        item.lockMode = "slot";
        playerInventory.setItem(player.selectedSlotIndex, item);

        //Spawns the backpack container entity.
        const backpack = backpackEventManager.spawnBackpackEntity(player, item);

        //Load the saved inventory on the backpack.
        backpackEventManager.loadInventory(backpack, item);
    }

    //Play sound effects.
    player.playSound("bundle.insert", { pitch: 1.0, volume: 1.0 });
});

backpackEventManager.on("inventoryOpen", ({ player, item }) => {
    const playerInventory = player.getComponent("minecraft:inventory").container;

    const itemData = item.getDynamicProperty("itemData");
    const parsedItemData = itemData ? JSON.parse(itemData) : null;
    const itemCustomId = parsedItemData ? parsedItemData.customId : null;

    for (let slot = 0; slot < playerInventory.size; slot++) {
        const currentItem = playerInventory.getItem(slot);

        if (currentItem) {
            const currentItemData = currentItem.getDynamicProperty("itemData");
            const parsedCurrentItemData = currentItemData ? JSON.parse(currentItemData) : null;
            const currentCustomId = parsedCurrentItemData ? parsedCurrentItemData.customId : null;

            if (config.prohibitedItems.includes(currentItem.typeId)) {
                if (currentItemData) {
                    //Unlocks the backpack into the current slot.
                    if (currentCustomId !== itemCustomId) {
                        //If item has the property "itemData" and doesn't have the same customId as the selected backpack, lock it.
                        currentItem.lockMode = "inventory";
                        playerInventory.setItem(slot, currentItem);
                    }
                } else {
                    //If item doesn't have the property "itemData", lock it.
                    currentItem.lockMode = "inventory";
                    playerInventory.setItem(slot, currentItem);
                }
            }
        }
    }
});

backpackEventManager.on("inventoryClose", ({ player, item }) => {
    const playerInventory = player.getComponent("minecraft:inventory").container;

    const itemData = item.getDynamicProperty("itemData");
    const parsedItemData = itemData ? JSON.parse(itemData) : null;
    const itemCustomId = parsedItemData ? parsedItemData.customId : null;

    for (let slot = 0; slot < playerInventory.size; slot++) {
        const currentItem = playerInventory.getItem(slot);

        if (currentItem) {
            const currentItemData = currentItem.getDynamicProperty("itemData");
            const parsedCurrentItemData = currentItemData ? JSON.parse(currentItemData) : null;
            const currentCustomId = parsedCurrentItemData ? parsedCurrentItemData.customId : null;

            if (config.prohibitedItems.includes(currentItem.typeId)) {
                if (currentItemData) {
                    //Unlocks the backpack into the current slot.
                    if (currentCustomId !== itemCustomId) {
                        //If item has the property "itemData" and doesn't have the same customId as the selected backpack, unlock it.
                        currentItem.lockMode = "none";
                        playerInventory.setItem(slot, currentItem);
                    }
                } else {
                    //If item doesn't have the property "itemData", unlock it.
                    currentItem.lockMode = "none";
                    playerInventory.setItem(slot, currentItem);
                }
            }
        }
    }
});

// ---------------------------------
// World Events
// ---------------------------------

//When player leaves in multiplayer when holding a backpack, despawn the backpack container.
world.beforeEvents.playerLeave.subscribe(({ player }) => {
    for (const entity of world.getDimension("overworld").getEntities({ type: "peepss:backpack_container" })) {
        if (entity.getDynamicProperty("owner") == player.name ) {
            entity.triggerEvent("minecraft:despawn")
        }
    }
});

//When player leaves and joins the world again in singleplayer when holding a backpack, despawn the backpack container.
world.afterEvents.playerSpawn.subscribe(({ player }) => {
    for (const entity of world.getDimension("overworld").getEntities({ type: "peepss:backpack_container" })) {
        if (entity.getDynamicProperty("owner") == player.name ) {
            entity.triggerEvent("minecraft:despawn")
        }
    }
});
